#!/usr/bin/python
# -*- coding: utf-8 -*-
#####
# Fichero de variables
# Guarda todos los Variables relacionados con los Variables que maneja el programa
####

#Copyright (C) 2006  Inmaculada Luengo Merino, Nstor Arocha Rodrguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


class Variable:
	"""
	Variable generica. De esta clase heredan las Variables usados en el programa. Define los elementos comnes a todas las variables y posee las funciones de acceso genricas
	"""


	def __init__(self,nombre,valores,valorpordefecto,descripcion):
		#VARIABLES PUBLICAS
		self.nombre=nombre
		self.valorpordefecto=valorpordefecto
		self.descripcion=descripcion
		if valores == None: self.valores=dict()
		else: self.valores=valores

	#FUNCIONES PUBLICAS
		
	def update(self,objeto):
		"""Funcin de copia de los atributos para cualquier variable.
		Sirve para hacer copias entre variables
		"""
		self.nombre=objeto.nombre
		self.descripcion=objeto.descripcion
		if objeto.valores:
			self.valores=objeto.valores.copy()

	def __setitem__(self,indice,valor):
		"""Definicion del operador []=. Permite modificar los atributos de la variable a travs de un indice numrico"""
		if indice==0:
			self.nombre=valor
		elif indice==3:
			self.valorpordefecto=valor
		elif indice==4:
			self.descripcion=valor

	def __getitem__(self,indice):
		"""Definicion del operador[]. Permite acceder a los atributos de la variable por indices numricos"""
		if indice==0:
			return self.nombre
		elif indice==3:
			return self.valorpordefecto
		elif indice==4:
			return self.descripcion
	
	def ristra(self,valor):
		"""Por defecto, el valor interno se convierte a ristra con repr. Solo algunas de las clases herederas necesitaran modificarla"""
		return repr(valor)



class Entero(Variable):
	"""
	La clase entero. Almacena cualquier nmero entero, garantizando las operaciones de transformacin a formato interno y a ristra. Hereda de variable
	"""
	def __init__(self,nombre=None,valores=None,valorpordefecto=0,descripcion=None):
		"""
		Inicializacion de variable entera
		"""
		Variable.__init__(self,nombre,valores,valorpordefecto,descripcion)
		self.tipo="Entero"

	def interno(self,valor):
		"""
		transforma el valor al formato en el que se desea guardar, 
		comprobando si un valor dado se ajusta al tipo de variable
		Genera una excepcin en caso de que la conversin no funcione
		"""
		return int(valor)

class Real(Variable):
	"""
	La clase Real. Almacena cualquier nmero real, garantizando las operaciones de transformacin a formato interno y a ristra. Hereda de variable
	"""
	def __init__(self,nombre=None,valores=None,valorpordefecto=0,descripcion=None):
		"""Inicializacion del objeto Real"""
		Variable.__init__(self,nombre,valores,valorpordefecto,descripcion)
		self.tipo="Real"
	def interno(self,valor):
		"""
		transforma el valor al formato en el que se desea guardar, 
		comprobando si un valor dado se ajusta al tipo de variable
		Genera una excepcin en caso de que la conversin no funcione
		"""
		return float(valor)

class Booleano(Variable):
	"""La clase Booleano,representa a un dato que solo puede tener valor verdadero o falso. El almacenamiento en python corresponde a los valores True o False """
	def __init__(self,nombre=None,valores=None,valorpordefecto=None,descripcion=None):
		"""Inicializacin del objeto booleano"""
		Variable.__init__(self,nombre,valores,valorpordefecto,descripcion)
		self.tipo="Booleano"
	def interno(self,valor):
		"""
		transforma el valor al formato en el que se desea guardar, 
		"""
		if not valor:
			return False
		elif valor=="0":
			return False
		else:
			return True

	def ristra(self,valor):
		"""Devuelve una ristra que representa al valor. Puede ser "0"  "1" """
		if valor:
			return "1"
		else:
			return "0"

class Fecha(Variable):
	"""
	La clase Fecha. Almacena una fecha en formato DD/MM/YYYY, garantizando las operaciones de transformacin a formato interno y a ristra. Hereda de variable
	"""
	def __init__(self,nombre=None,valores=None,valorpordefecto=None,descripcion=None):
		"""Inicializacion de fecha"""
		if valorpordefecto==None:
			valorpordefecto="1/1/1970"
		Variable.__init__(self,nombre,valores,valorpordefecto,descripcion)
		self.tipo="Fecha"

	def interno(self,valor):
		"""Pasa la fecha a formato interno, que es el nmero de segundos a partir del 1 de enero de 1970 TODO"""
		import datetime
		array=valor.split("/")
		fecha=datetime.date(int(array[2]),int(array[1]),int(array[0]))
		import time
		return time.mktime(fecha.timetuple())

	def ristra(self,valor):
		"""Pasa del formato interno a una ristra de la forma ""DD/MM/YYYY" y la devuelve """
		import time
		import datetime
		fecha=datetime.datetime.fromtimestamp(valor)
		return repr(fecha.day)+"/"+repr(fecha.month)+"/"+repr(fecha.year)



